'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ListType = exports.ListJSONType = exports.List = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _tcomb = require('tcomb');

var _uri = require('rheactor-value-objects/uri');

var _uri2 = _interopRequireDefault(_uri);

var _model = require('./model');

var _link = require('./link');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var $context = new _uri2.default('https://github.com/ResourcefulHumans/staRHs-models#List');
var PositiveIntegerType = (0, _tcomb.refinement)(_tcomb.Number, function (n) {
  return n > 0 && n % 1 === 0;
}, 'PositiveIntegerType');
var ZeroOrPositiveIntegerType = (0, _tcomb.refinement)(_tcomb.Number, function (n) {
  return n >= 0 && n % 1 === 0;
}, 'ZeroOrPositiveIntegerType');
var ModelListType = (0, _tcomb.list)(_model.Model);
var LinkListType = (0, _tcomb.list)(_link.Link);

var List = exports.List = function () {
  /**
   * @param {Array<Model>} items
   * @param {number} total
   * @param {number} itemsPerPage
   * @param {Array<Link>} links
   */
  function List(items, total, itemsPerPage) {
    var links = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

    _classCallCheck(this, List);

    ModelListType(items);
    ZeroOrPositiveIntegerType(total);
    PositiveIntegerType(itemsPerPage);
    LinkListType(links);
    this.$context = $context;
    this.$links = links;
    this.items = items;
    this.total = total;
    this.itemsPerPage = itemsPerPage;
    this.hasNext = this.$links.filter(function (link) {
      return link.rel === 'next';
    }).length > 0;
    this.hasPrev = this.$links.filter(function (link) {
      return link.rel === 'prev';
    }).length > 0;
  }

  /**
   * @returns {{$context: string, $links: Array<object>, items: Array<object>, total: number, itemsPerPage: number, hasNext: boolean, hasPrev: boolean}}
   */


  _createClass(List, [{
    key: 'toJSON',
    value: function toJSON() {
      var d = {
        $context: this.$context.toString(),
        items: this.items.map(function (item) {
          return item.toJSON();
        }),
        total: this.total,
        itemsPerPage: this.itemsPerPage,
        hasNext: this.hasNext,
        hasPrev: this.hasPrev
      };
      if (this.$links.length) d.$links = this.$links.map(function (link) {
        return link.toJSON();
      });
      return d;
    }

    /**
     * @param {{$context: string, $links: Array<object>, items: Array<object>, total: number, itemsPerPage: number, hasNext: boolean, hasPrev: boolean}} data
     * @param {function} transformer item transformer
     * @returns {List}
     */

  }], [{
    key: 'fromJSON',
    value: function fromJSON(data, transformer) {
      ListJSONType(data);
      (0, _tcomb.Function)(transformer);
      return new List(data.items.map(transformer), data.total, data.itemsPerPage, data.$links ? data.$links.map(_link.Link.fromJSON) : []);
    }

    /**
     * @returns {URIValue}
     */

  }, {
    key: '$context',
    get: function get() {
      return $context;
    }
  }]);

  return List;
}();

var ListJSONType = exports.ListJSONType = (0, _tcomb.struct)({
  $context: (0, _tcomb.refinement)(_tcomb.String, function (s) {
    return s === $context.toString();
  }, 'ListContext'),
  $links: (0, _tcomb.list)(_link.LinkJSONType),
  total: ZeroOrPositiveIntegerType,
  itemsPerPage: PositiveIntegerType
}, 'ListJSONType');
var ListType = exports.ListType = (0, _tcomb.irreducible)('ListType', function (x) {
  return x instanceof List;
});